/**
* TcHmiGrid
* -------------------------------------------------------------------
* (c) 2016 BECKHOFF Automation GmbH & Co. KG http://www.beckhoff.com
* -------------------------------------------------------------------
* @ignore
*/
/**
* @namespace TcHmi
*/
declare module TcHmi {
    /**
    * @namespace TcHmi.Controls
    */
    module Controls {
        /**
        * @namespace TcHmi.Controls.System
        */
        module System {
            /**
            *
            * @class TcHmiGrid
            */
            class TcHmiGrid extends TcHmi.Controls.System.TcHmiContainerControl {
                /**
                 * Constructor
                 * @param element
                 * @param attrs
                 * @param pcElement
                 * @param pcAttrs
                 */
                constructor(element: JQuery, pcElement: JQuery, attrs: TcHmi.Controls.ControlAttributeList);
                /** Reference to the root dom element of the current control template as  jquery object.
                @member */
                private __elementTemplateRoot;
                /** Main Gridcontainer of this tchmigrid */
                private __elementGridcontainer;
                /**
                * Internal reference to the attribute 'data-tchmi-cell-wrap'
                * @member
                */
                protected __cellWrap: boolean | undefined;
                /**
                * Internal reference to the attribute 'data-tchmi-cell-options'
                * @member
                */
                protected __cellOptions: TcHmiGrid.ICellOptions[] | undefined;
                /**
                * Internal reference to the attribute 'data-tchmi-row-options'
                * @member
                */
                protected __rowOptions: TcHmiGrid.IRowOptions[] | undefined;
                /**
                * Internal reference to the attribute 'data-tchmi-column-options'
                * @member
                */
                protected __columnOptions: TcHmiGrid.IColumnOptions[] | undefined;
                /** Holds the pixel size the column uses, null if all relative. Cache has to be rebuild if variable is null */
                protected __cacheWidthPerColumn: (null | number)[] | null;
                /** Holds the pixel size the row uses, null if all relative. Cache has to be rebuild if variable is null */
                protected __cacheHeightPerRow: (null | number)[] | null;
                /** Cache has to be rebuild if variable is null */
                private __cacheCellHeightPerRow;
                private __onControlGridRowIndexChangedDestroyEvent;
                private __onControlGridColumnIndexChangedDestroyEvent;
                private __onControlsMovedDestroyEvent;
                private __onControlsResizedDestroyEvent;
                private __onControlPositionParameterChangedDestroyEvent;
                private __onControlSizeParameterChangedDestroyEvent;
                /**
                 * If raised, the control object exists in control cache and constructor of each inheritation level was called.
                 * Call attribute processor functions here to initialize default values!
                 * @function __previnit
                 */
                __previnit(): void;
                /**
                * If raised, all attributes have been set to it's default or dom values.
                * @function __init
                */
                __init(): void;
                /**
                * Is called by tachcontrol() after the control instance gets part of the current DOM.
                * Is only allowed to be called from the framework itself!
                * @function __attach
                */
                __attach(): void;
                /**
                * Is called by tachcontrol() after the control instance is no longer part of the current DOM.
                * Is only allowed to be called from the framework itself!
                * @function __detach
                */
                __detach(): void;
                /**
                * Destroy the current control instance.
                * Will be called automatically if system destroys control!
                * @function destroy
                */
                destroy(): void;
                /**
                 * Adds a child to the Grid
                 * @function
                 */
                addChild(co: TcHmi.Controls.System.baseTcHmiControl, pos: number | null): void;
                __addChild(co: TcHmi.Controls.System.baseTcHmiControl, pos: number | null): void;
                /**
                 * Remove a child control.
                 * @function
                 */
                removeChild(co: TcHmi.Controls.System.baseTcHmiControl): void;
                __removeChild(co: TcHmi.Controls.System.baseTcHmiControl): void;
                /**
                 * Moves a control to the correct Grid cell
                 */
                private __onGridRowOrColumnIndexChanged();
                /**
                Adjust Cell Dimension on Control resize or move
                @param move Is this a moved control? If not it was resized
                @function */
                private __onControlGeometryChanged(move);
                /**
                    calls processHeight if needed
                    @return true if a change was detected
                 */
                private __rebuildCellHeightPerRowCache();
                /**
                * Returns the current cell wrap value.
                * @function
                * @retuns   {number}    The current cell wrap value.
                */
                getCellWrap(): boolean | undefined;
                /**
                 * Sets a new cell wrap
                 * @param valueNew
                 */
                setCellWrap(valueNew: boolean | null): void;
                /**
                 *
                 *
                 */
                protected __processCellWrap(): void;
                /**
                * Returns the current width mode of the control.
                * @function
                * @returns   {string}    The current width mode value.
                */
                getWidthMode(): "Value" | "Parent" | "Content" | undefined;
                /**
                * Updates caches...
                * @function
                */
                protected __processWidthMode(): void;
                /**
                * Returns the current height mode of the control.
                * @function
                * @returns   {string}    The current height mode value.
                */
                getHeightMode(): "Value" | "Parent" | "Content" | undefined;
                /**
                * Updates caches...
                * @function
                */
                protected __processHeightMode(): void;
                /**
                 * Adds new Rows if needed.
                 * If Rows are removed the affected controls are moved in the correct cell if needed.
                 */
                __recalculateRowCount(): void;
                /**
                 * Checks each Row in the Grid and adds/removes cells if needed.
                 * After that all controls are moved in the correct cell if needed.
                 */
                __recalculateColumnCount(): void;
                /**
                * Returns the current row options value.
                * @function
                * @returns The current row options value.
                */
                getRowOptions(): TcHmiGrid.IRowOptions[] | undefined;
                /**
                 * Sets new Row options
                 * @param valueNew
                 */
                setRowOptions(valueNew: TcHmiGrid.IRowOptions[] | null): void;
                /**
                 * Processes row options
                 * height is set directly
                 * min/max value are set after comparing to the content dimension
                 */
                protected __processRowOptions(): void;
                /**
                * Returns the current column options value.
                * @function
                * @returns The current column options value.
                */
                getColumnOptions(): TcHmiGrid.IColumnOptions[] | undefined;
                /**
                 * Sets new column options
                 * @param valueNew
                 */
                setColumnOptions(valueNew: TcHmiGrid.IColumnOptions[] | null): void;
                /**
                 * Processes column options
                 * width is set directly
                 * min/max value are set after comparing to the content dimension
                 */
                protected __processColumnOptions(): void;
                /**
                 * PaddingTop
                 */
                getCellOptions(): TcHmiGrid.ICellOptions[] | undefined;
                setCellOptions(valueNew: TcHmiGrid.ICellOptions[] | null): void;
                protected __processCellOptions(): void;
                /**
                * Moves all Controls to the correct Grid Container.
                */
                private __correctAllGridPositions();
                /**
                * Moves the Control to the correct Grid Container
                * @param co
                * @param pos
                */
                private __correctGridPosition(co, pos?);
                /**
                 * Append a number of Rows
                 * If the number is undefined or null one row is created
                 * @param num Number of Rows to create. If undefined or null one row is created
                 * @return JQuery Object of all new Rows (last Element in JQuery is last one in DOM) or a empty JQuery Object
                 */
                private __appendRows(num?);
                /**
                 * Remove Rows from DOM and returns it as a JQuery Obj
                 * If the index is undefined the last row is deleted
                 *
                 * If the index is not valid a empty JQuery is returned.
                 * @param num Number of Rows to delete. If undefined or null one row is created
                 * @return JQuery Object of the deleted Row or a empty JQuery Object
                 */
                private __removeRows(num?);
                /**
                 * Creates columns till the Grid is fulfilled
                 * @param targetRow
                 * @return JQuery Object of the new Cell or a empty JQuery Object
                 */
                private __createColumns(targetRow);
                /**
                 * Creates a new Cell and appends it to the last DOM element in given JQuery Object
                 * @param targetRow
                 * @return JQuery Object of the new Cell or a empty JQuery Object
                 */
                private __createCell(targetRow);
                /**
                 * Processes column / row options
                 * width/height is set directly
                 * min/max value are set after comparing to the content dimension
                 * @param direction
                 */
                private __processSequenceOptions(direction);
                /**
                 * Returns the calculated width in pixel if self defined (not percent based) or based on the children.
                 */
                __getContentWidth(): number | null;
                /**
                 * Returns the calculated height in pixel if self defined (not percent based) or based on the children.
                 */
                __getContentHeight(): number | null;
                /**
                * Processes the current height and height unit.
                * @function
                */
                __processHeight(callerControl?: TcHmiControl): void;
                /**
                * Processes the current width and width unit.
                * @function
                */
                __processWidth(callerControl?: TcHmiControl): void;
                /**
                 * Gets the pixel value of a style or zero.
                 * Could be wrong if the style is overwritten!
                 */
                private static __getPixelNumberInStyle(cellStyle, longhandStylename, shorthandStylename?);
            }
            module TcHmiGrid {
                interface ISequenceOption {
                    /** Defines size of the column/row of the grid. Can be a pixel value (42px), percent value (42%), a grow factor (1) or 'auto' (equivalent to grow factor 1). The grow factor determines how much the items will grow relative to the rest of the items when positive free space is distributed. */
                    size: number | null;
                    sizeUnit: TcHmi.DimensionUnit | 'factor';
                    sizeMode: 'Value' | 'Content';
                    minSize: number | null;
                    minSizeUnit: TcHmi.DimensionUnit;
                    maxSize: number | null;
                    maxSizeUnit: TcHmi.DimensionUnit;
                    overflow: boolean;
                }
                interface IColumnOptions {
                    width: number | null;
                    widthUnit: TcHmi.DimensionUnit | 'factor';
                    widthMode: 'Value' | 'Content';
                    minWidth: number | null;
                    minWidthUnit: TcHmi.DimensionUnit;
                    maxWidth: number | null;
                    maxWidthUnit: TcHmi.DimensionUnit;
                    overflow: boolean;
                }
                interface IRowOptions {
                    height: number | null;
                    heightUnit: TcHmi.DimensionUnit | 'factor';
                    heightMode: 'Value' | 'Content';
                    minHeight: number | null;
                    minHeightUnit: TcHmi.DimensionUnit;
                    maxHeight: number | null;
                    maxHeightUnit: TcHmi.DimensionUnit;
                    overflow: boolean;
                }
                interface ICellOptions {
                    rowIndex: number;
                    columnIndex: number;
                    borderWidth?: TcHmi.BorderWidth;
                    borderStyle?: TcHmi.BorderStyle;
                    borderColor?: TcHmi.SolidColor;
                    padding?: TcHmi.FourSidedCss;
                    backgroundColor?: TcHmi.Color;
                    backgroundImage?: string;
                    backgroundImagePadding?: TcHmi.FourSidedCss;
                    backgroundImageWidth?: number;
                    backgroundImageWidthUnit?: TcHmi.DimensionUnit;
                    backgroundImageHeight?: number;
                    backgroundImageHeightUnit?: TcHmi.DimensionUnit;
                    backgroundImageHorizontalAlignment?: TcHmi.HorizontalAlignment;
                    backgroundImageVerticalAlignment?: TcHmi.VerticalAlignment;
                }
                const enum Direction {
                    column = 0,
                    row = 1,
                }
            }
        }
    }
}
